# Generated by Django 2.2.7 on 2020-05-03 18:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('test_app_github', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='motorbike',
            name='price',
            field=models.FloatField(db_index=True),
        ),
    ]
